' Example 3a:  Write then verify an encrypted secret on a SmartDongle 
'
' MicroWorks, Inc.
' 2808 North Cole Road
' Boise, ID 83704
'
' http://www.smartdongle.com

Imports System
Imports System.Security.Cryptography
Imports System.Text
Imports System.IO

Module Example2a

    Sub Main()

        Dim dongle As New SmartDongle()
        Dim dnglError As SmartDongle.SmartDnglError


        '  Your keys here
        Dim P1 As Int64 = &HEC6CC589AEFD1E75
        Dim P2 As Int64 = &HFCEC0A6A82747B3F


        ' Your secret data
        Dim your_secret As String = "This is the secret written to SmartDongle, it can be passwords, serial numbers, program parameters, etc"


        ' Padding is zeroes and CipherMode has been set to
        ' Electronic Codebook (ECB) to match the algorithm in the C examples.
        '
        ' The AES key and user secret below are for example only, the user
        ' should create a new key and secret for their application.
        '
        '  A System.Security.Cryptography.CryptographicException will
        '  occur if illegal AES key length or AES key size are given below.

        Dim aes_key As String = "7obdgEn2'TH>]SE|"
        Dim aes_KeySize As Integer = 128
        Dim aes_BlockSize As Integer = 128
        Dim RijndaelAlg As Rijndael
        Dim rgbKey() As Byte
        Dim rgbIV() As Byte
        Dim encrypt As ICryptoTransform
        Dim decrypt As ICryptoTransform
        Dim plainText() As Byte
        Dim cipherText() As Byte
        Dim result() As Byte

Try
        RijndaelAlg = Rijndael.Create()
        RijndaelAlg.KeySize = aes_KeySize
        RijndaelAlg.BlockSize = aes_BlockSize
        RijndaelAlg.Mode = CipherMode.ECB
        RijndaelAlg.Padding = PaddingMode.Zeros

        ' Initial vector (rgbIV) is the same as the key as in the C example code 
        Dim encoding As System.Text.ASCIIEncoding = New System.Text.ASCIIEncoding()
        rgbKey = encoding.GetBytes(aes_key)
        rgbIV = rgbKey

        encrypt = RijndaelAlg.CreateEncryptor(rgbKey, rgbIV)
        decrypt = RijndaelAlg.CreateDecryptor(rgbKey, rgbIV)

        ' Convert secret into byte array, then encrypt
        plainText = encoding.GetBytes(your_secret)
        cipherText = encrypt.TransformFinalBlock(plainText, 0, plainText.Length)

Catch ex As System.Security.Cryptography.CryptographicException
        MsgBox("Failed to initialized Rijndael Encryption / Decryption Objects")

        ' handle error
        Return
End Try

        ' Write the encrypted secret to the begining of SmartDongle user memory.
        '
        ' Note: The number of writes is limited, SmartDongle uses a Catalyst
        '       CAT25C256 SPI eprom, which has a life expectancy of approximately
        '       1 Million writes.  Unlimited reads, of course.
        '

        Dim address As Int16 = 0 ' User memory address. Byte addressable from 0 to 32431
        dnglError = dongle.Write(P1, P2, address, cipherText, cipherText.Length)
        If dnglError <> SmartDongle.SmartDnglError.UskOk Then

            System.Console.WriteLine("Failed: Writing Secret to SmartDongle")
            System.Console.WriteLine(SmartDongle.GetErrorString(dnglError))

            ' handle error
            Return
        End If


        ' Do a data integrity check.
        ' Read the encrypted secret from the SmartDongle,
        ' decrypt and compare with original secret.
        Dim input(cipherText.Length - 1) As Byte
        dnglError = dongle.Read(P1, P2, address, input, input.Length)

        If dnglError <> SmartDongle.SmartDnglError.UskOk Then

            System.Console.WriteLine("Failed: Reading Secret back from SmartDongle")
            System.Console.WriteLine(SmartDongle.GetErrorString(dnglError))

            ' handle error
            Return
        End If


Try
        ' Decrypt
        result = decrypt.TransformFinalBlock(input, 0, input.Length)

Catch ex As System.Security.Cryptography.CryptographicException

        MsgBox("Failure of Rijndael decryption occurred")

        ' handle error
        Return
End Try

        ' Compare to original secret, using orignal secret length since the
        ' result will be a multiple of block length and padded with zeroes
        Dim i As Integer
        For i = 1 To plainText.Length - 1
            If (result(i) <> plainText(i)) Then
                Exit For
            End If
        Next

        If i = plainText.Length Then
            System.Console.WriteLine("Success: Wrote Secret to SmartDongle")
        Else
            System.Console.WriteLine("Failed: Error writing secret, data integrity check failed")
        End If

        System.Console.WriteLine("")
        System.Console.WriteLine("Press <Enter> to Exit")
        System.Console.ReadLine()
    End Sub

End Module
